% This program is used to plot steady-state simulations obtained from the
% the control and LVH hearts. The plotted figures corresponding to Figure
% 1A-F in Wu et al, PNAS, 106 (17), 7143 (2009).
% Fan Wu, MCW, 05/21/2008

clear;
setup;

global iPO2 idPsi iH_x iATP_x iADP_x iAMP_x iGTP_x iGDP_x iPI_x iNADH_x iQH2_x ...
       iOAA_x iACCOA_x iCIT_x iICIT_x iAKG_x iSCOA_x iCOASH_x iSUC_x iFUM_x ...
       iMAL_x iGLU_x iASP_x iK_x iMg_x iCO2tot_x iCred_i iATP_i iADP_i iAMP_i ...
       iPI_i iH_i iMg_i iK_i iATP_c iADP_c iPI_c iH_c iMg_c iK_c iPYR_x iPYR_i ...
       iPYR_c iCIT_i iCIT_c iAKG_i iAKG_c iSUC_i iSUC_c iMAL_i iMAL_c iASP_i ...
       iASP_c iGLU_i iGLU_c iFUM_i iFUM_c iICIT_i iICIT_c iPCr_c iAMP_c iCr_c;
     
global i_J_C1 i_J_C3 i_J_C4 i_J_F1 i_J_ANT i_J_PI1 i_J_Hle i_J_KH i_J_pdh i_J_cits ...
       i_J_acon i_J_isod i_J_akgd i_J_scoas i_J_sdh i_J_fum i_J_mdh i_J_ndk i_J_got ...
	     i_J_PI2 i_J_ATPt i_J_ADPt i_J_AMPt i_J_PYR_H i_J_GLU_H i_J_CIT_MAL i_J_SUC_MAL ... 
       i_J_AKG_MAL i_J_MAL_PI i_J_ASP_GLU i_J_PYRt i_J_CITt i_J_ICITt i_J_MALt ... 
	     i_J_AKGt i_J_SUCt i_J_FUMt i_J_GLUt i_J_ASPt i_J_CKe i_J_AKe i_J_AtC;

% Loading results obtained from the simulation of cardiac energetics under
% physiological conditions
load steady_states_control.mat;
MVO2_sim_con = MVO2_sim;
PCrATP_sim1_con = PCrATP_sim1;
PIPCr_sim_con = PIPCr_sim;
dG_AtC_con = Flux_st(:,end);
PI_c_con = PI_c;
ADP_c_con = ADP_c;

% Loading results obtained from the simulation of cardiac energetics of the
% moderate LVH hearts
load steady_states_LVH.mat;
MVO2_sim_LVH = MVO2_sim;
PCrATP_sim1_LVH = PCrATP_sim1;
PIPCr_sim_LVH = PIPCr_sim;
dG_AtC_LVH = Flux_st(:,end);
PI_c_LVH = PI_c;
ADP_c_LVH = ADP_c;

% Loading results obtained from the simulation of cardiac energetics of the
% early-stage LVH hearts
load steady_states_mLVH.mat;
MVO2_sim_mLVH = MVO2_sim;
PCrATP_sim1_mLVH = PCrATP_sim1;
PIPCr_sim_mLVH = PIPCr_sim;
dG_AtC_mLVH = Flux_st(:,end);
PI_c_mLVH = PI_c;
ADP_c_mLVH = ADP_c;

load HEP_MVO2.txt;
MVO2_st_z = HEP_MVO2(:,1);
MVO2_st_e = HEP_MVO2(:,2);
PCrATP_st_z = HEP_MVO2(:,3);
PCrATP_st_e = HEP_MVO2(:,4);
PIPCr_st_z = HEP_MVO2(:,5);
PIPCr_st_e = HEP_MVO2(:,6);


%% Plotting the results for the paper
% load results_steady_states_CKKO.mat;

figure(4021); clf; 
axes('position',[0.15 0.15 0.75 0.75]);
hold on; box on;
% Defining the bounds of x-axis and y-axis
xlb = 0; xub = 13; ylb = 0; yub = 3.0;
plot(MVO2_sim_con,PCrATP_sim1_con,'k-','LineWidth',1.5); 
% Labeling the graphs
% Zero J_AtC
plot([MVO2_sim_con(1) MVO2_sim_con(1)],[ylb yub],'k:','LineWidth',1.0);
% Baseline J_AtC
plot([MVO2_BL MVO2_BL],[ylb yub],'k:','LineWidth',1.0);
% Maximum J_AtC
plot([max(MVO2_st_z) max(MVO2_st_z)],[ylb yub],'k:','LineWidth',1.0);
% Display J_AtC mmol s-1 (l cell)-1
text((xub-xlb)*0.05+xlb,(yub-ylb)*0.9+ylb,'A','FontSize',24);
text((xub-xlb)*0.04+xlb,(yub-ylb)*0.28+ylb,'\it{J_{ATPase}}','FontSize',16);
text((xub-xlb)*0.04+xlb,(yub-ylb)*0.20+ylb,'= 0','FontSize',16);
text((xub-xlb)*0.28+xlb,(yub-ylb)*0.20+ylb,'0.36','FontSize',16);
text((xub-xlb)*0.75+xlb,(yub-ylb)*0.20+ylb,'1.2','FontSize',16);
for i=1:length(MVO2_st_z)
  % You can use your dateset instead of MVO2 and PCrATP
  px = MVO2_st_z(i); pxe = MVO2_st_e(i); % Loading x-axis data and errors
  py = PCrATP_st_z(i); pye = PCrATP_st_e(i); % Loading y-axis data and errors
  % Defining widith or errorbars
  pxd = (xub-xlb)*0.015; pyd = (yub-ylb)*0.015; 
  % Plotting crosslines 
  plot([px px],[py-pye py+pye],'k-');
  plot([px-pxe px+pxe],[py py],'k-');
  % Plotting errorbars
  plot([px-pxd px+pxd],[py+pye py+pye],'k-');
  plot([px-pxd px+pxd],[py-pye py-pye],'k-');
  plot([px-pxe px-pxe],[py-pyd py+pyd],'k-');
  plot([px+pxe px+pxe],[py-pyd py+pyd],'k-');
end
% Finally plotting data points
% Zhang et al, AJP, 2005
plot(MVO2_st_z(1:4),PCrATP_st_z(1:4),'ko',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Zhang et al, AJP, 2003
plot(MVO2_st_z(5),PCrATP_st_z(5),'k<',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Gong et al, AJP, 2003
plot(MVO2_st_z(6:7),PCrATP_st_z(6:7),'kd',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Ochiai et al, AJP 2001
plot(MVO2_st_z(8:12),PCrATP_st_z(8:12),'k^',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Gong et al, AJP 1999
plot(MVO2_st_z(13:14),PCrATP_st_z(13:14),'kv',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Bache et al, Cardio Res, 1999
plot(MVO2_st_z(15:18),PCrATP_st_z(15:18),'ks',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Setting axis properties
axis([xlb xub ylb yub]);
set(gca,'XtickLabel',[]);
set(gca,'fontsize',18); 
set(gca,'Xtick',0:2:12);
set(gca,'Ytick',ylb:0.5:yub);
ylabel('CrP/ATP'); 
% xlabel('MVO_2 (\mumol min^{-1} (g tissue)^{-1})');

%
figure(4022); clf;
axes('position',[0.15 0.15 0.75 0.75]);
hold on; box on;
xlb = 0; xub = 13; ylb = -0.025; yub = 0.5;
plot(MVO2_sim_con,PIPCr_sim_con,'k-','LineWidth',1.5); 
% Labeling the graphs
% Zero J_AtC
plot([MVO2_sim_con(1) MVO2_sim_con(1)],[ylb yub],'k:','LineWidth',1.0);
% Baseline J_AtC
plot([MVO2_BL MVO2_BL],[ylb yub],'k:','LineWidth',1.0);
% Maximum J_AtC
plot([max(MVO2_st_z) max(MVO2_st_z)],[ylb yub],'k:','LineWidth',1.0);
text((xub-xlb)*0.05+xlb,(yub-ylb)*0.9+ylb,'B','FontSize',24);
% plot(MVO2_st_z,PIPCr_inp,'r+');  
% Modify the baseline value of PI/ATP to represent its actual meaning
PIPCr_st_z([1 3:6 8 9 11 13:16]) = (1e-3/CrPBL_cellwater)/2; % average value of the detectable PI/PCr 0.0708
PIPCr_st_e([1 3:6 8 9 11 13:16]) = (1e-3/CrPBL_cellwater)/2;
for i=1:length(MVO2_st_z)
  % You can use your dateset instead of MVO2 and PCrATP
  px = MVO2_st_z(i); pxe = MVO2_st_e(i); % Loading x-axis data and errors
  py = PIPCr_st_z(i); pye = PIPCr_st_e(i); % Loading y-axis data and errors
  % Defining widith or errorbars
  pxd = (xub-xlb)*0.015; pyd = (yub-ylb)*0.015; 
  % Plotting crosslines 
  plot([px px],[py-pye py+pye],'k-');
  plot([px-pxe px+pxe],[py py],'k-');
  % Plotting errorbars
  plot([px-pxd px+pxd],[py+pye py+pye],'k-');
  plot([px-pxd px+pxd],[py-pye py-pye],'k-');
  plot([px-pxe px-pxe],[py-pyd py+pyd],'k-');
  plot([px+pxe px+pxe],[py-pyd py+pyd],'k-');
end 
% plot(MVO2_st_z,PIPCr_inp,'r+');
% plot(MVO2_st_z,PIPCr_st_z,'ks',...
%     'Markersize',8,'MarkerFaceColor',[1 1 1]);
plot(MVO2_st_z([1 3 4]),PIPCr_st_z([1 3 4]),'ko',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
plot(MVO2_st_z(2),PIPCr_st_z(2),'ko',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Zhang et al, AJP, 2003
plot(MVO2_st_z(5),PIPCr_st_z(5),'k<',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
% Gong et al, AJP, 2003
plot(MVO2_st_z(6),PIPCr_st_z(6),'kd',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
plot(MVO2_st_z(7),PIPCr_st_z(7),'kd',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Ochiai et al, AJP 2001
plot(MVO2_st_z([8 9 11]),PIPCr_st_z([8 9 11]),'k^',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
plot(MVO2_st_z([10 12]),PIPCr_st_z([10 12]),'k^',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Gong et al, AJP 1999
plot(MVO2_st_z(13:14),PIPCr_st_z(13:14),'kv',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
% Bache et al, Cardio Res, 1999
plot(MVO2_st_z([15 16]),PIPCr_st_z([15 16]),'ks',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
plot(MVO2_st_z(17:18),PIPCr_st_z(17:18),'ks',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Setting axis properties
axis([xlb xub ylb yub]);
set(gca,'fontsize',18);
set(gca,'Xtick',0:2:12);
set(gca,'Ytick',0:0.1:0.5); %set(gca,'Ytick',ylb:0.1:yub);
set(gca,'XtickLabel',[]);
% xlabel('MVO_2 (\mumol min^{-1} (g tissue)^{-1}');
ylabel('\DeltaPi/CrP'); 

%%
figure(4023); clf; 
axes('position',[0.15 0.15 0.75 0.75]);
hold on; box on;
% Defining the bounds of x-axis and y-axis
xlb = 0; xub = 13; ylb = 0; yub = 3.0; 
plot(MVO2_sim_mLVH,PCrATP_sim1_mLVH,'k-','LineWidth',1.5); 
% Labeling the graphs
% Zero J_AtC
plot([MVO2_sim_mLVH(1) MVO2_sim_mLVH(1)],[ylb yub],'k:','LineWidth',1.0);
% Baseline J_AtC
plot([MVO2_BL MVO2_BL],[ylb yub],'k:','LineWidth',1.0);
% Maximum J_AtC
plot([max(MVO2_st_z) max(MVO2_st_z)],[ylb yub],'k:','LineWidth',1.0);
% Display J_AtC mmol s-1 (l cell)-1
text((xub-xlb)*0.05+xlb,(yub-ylb)*0.9+ylb,'C','FontSize',24);
for i=5:7
  % You can use your dateset instead of MVO2 and PCrATP
  px = MVO2_st_LVH_z(i); pxe = MVO2_st_LVH_e(i); % Loading x-axis data and errors
  py = PCrATP_st_LVH_z(i); pye = PCrATP_st_LVH_e(i); % Loading y-axis data and errors
  % Defining widith or errorbars
  pxd = (xub-xlb)*0.015; pyd = (yub-ylb)*0.015; 
  % Plotting crosslines 
  plot([px px],[py-pye py+pye],'k-');
  plot([px-pxe px+pxe],[py py],'k-');
  % Plotting errorbars
  plot([px-pxd px+pxd],[py+pye py+pye],'k-');
  plot([px-pxd px+pxd],[py-pye py-pye],'k-');
  plot([px-pxe px-pxe],[py-pyd py+pyd],'k-');
  plot([px+pxe px+pxe],[py-pyd py+pyd],'k-');
end  
% Finally plotting data points
% LVH data from Bache et al, AJP-Heart, 1994
plot(MVO2_st_LVH_z(5:7),PCrATP_st_LVH_z(5:7),'ko',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Setting axis properties
axis([xlb xub ylb yub]);
set(gca,'XtickLabel',[]);
set(gca,'fontsize',18); 
set(gca,'Xtick',0:2:12);
set(gca,'Ytick',ylb:0.5:yub);
ylabel('CrP/ATP'); 
% xlabel('MVO_2 (\mumol min^{-1} (g tissue)^{-1})');

%
figure(4024); clf;
axes('position',[0.15 0.15 0.75 0.75]);
hold on; box on;
xlb = 0; xub = 13; ylb = -0.025; yub = 0.5;
plot(MVO2_sim_mLVH,PIPCr_sim_mLVH,'k-','LineWidth',1.5);
% Labeling the graphs
% Zero J_AtC
plot([MVO2_sim_mLVH(1) MVO2_sim_mLVH(1)],[ylb yub],'k:','LineWidth',1.0);
% Baseline J_AtC
plot([MVO2_BL MVO2_BL],[ylb yub],'k:','LineWidth',1.0);
% Maximum J_AtC
plot([max(MVO2_st_z) max(MVO2_st_z)],[ylb yub],'k:','LineWidth',1.0);
text((xub-xlb)*0.05+xlb,(yub-ylb)*0.9+ylb,'D','FontSize',24);
% plot(MVO2_st_z,PIPCr_inp,'r+');  
% Modify the baseline value of PI/ATP to represent its actual meaning
PIPCr_st_LVH_z([5]) = (1e-3/CrPBL_cellwater_mLVH)/2; % average value of the detectable PI/PCr 0.0708
PIPCr_st_LVH_e([5]) = (1e-3/CrPBL_cellwater_mLVH)/2;
for i=5:7
  % You can use your dateset instead of MVO2 and PCrATP
  px = MVO2_st_LVH_z(i); pxe = MVO2_st_LVH_e(i); % Loading x-axis data and errors
  py = PIPCr_st_LVH_z(i); pye = PIPCr_st_LVH_e(i); % Loading y-axis data and errors
  % Defining widith or errorbars
  pxd = (xub-xlb)*0.015; pyd = (yub-ylb)*0.015; 
  % Plotting crosslines 
  plot([px px],[py-pye py+pye],'k-');
  plot([px-pxe px+pxe],[py py],'k-');
  % Plotting errorbars
  plot([px-pxd px+pxd],[py+pye py+pye],'k-');
  plot([px-pxd px+pxd],[py-pye py-pye],'k-');
  plot([px-pxe px-pxe],[py-pyd py+pyd],'k-');
  plot([px+pxe px+pxe],[py-pyd py+pyd],'k-');
end  
% LVH data from Bache et al, AJP-Heart, 1994
plot(MVO2_st_LVH_z(5),PIPCr_st_LVH_z(5),'ko',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
plot(MVO2_st_LVH_z(6:7),PIPCr_st_LVH_z(6:7),'ko',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Setting axis properties
axis([xlb xub ylb yub]);
set(gca,'fontsize',18);
set(gca,'Xtick',0:2:12);
set(gca,'Ytick',0:0.1:0.5); %set(gca,'Ytick',ylb:0.1:yub);
set(gca,'XtickLabel',[]);
% xlabel('MVO_2 (\mumol min^{-1} (g tissue)^{-1}');
ylabel('\DeltaPi/CrP'); 

%%
figure(4025); clf; 
axes('position',[0.15 0.15 0.75 0.75]);
hold on; box on;
% Defining the bounds of x-axis and y-axis
xlb = 0; xub = 13; ylb = 0; yub = 3.0;
plot(MVO2_sim_LVH,PCrATP_sim1_LVH,'k-','LineWidth',1.5); 
% Labeling the graphs
% Zero J_AtC
plot([MVO2_sim_LVH(1) MVO2_sim_LVH(1)],[ylb yub],'k:','LineWidth',1.0);
% Baseline J_AtC
plot([MVO2_BL MVO2_BL],[ylb yub],'k:','LineWidth',1.0);
% Maximum J_AtC
plot([max(MVO2_st_z) max(MVO2_st_z)],[ylb yub],'k:','LineWidth',1.0);
% Display J_AtC mmol s-1 (l cell)-1
text((xub-xlb)*0.05+xlb,(yub-ylb)*0.9+ylb,'E','FontSize',24);
for i=1:4
  % You can use your dateset instead of MVO2 and PCrATP
  px = MVO2_st_LVH_z(i); pxe = MVO2_st_LVH_e(i); % Loading x-axis data and errors
  py = PCrATP_st_LVH_z(i); pye = PCrATP_st_LVH_e(i); % Loading y-axis data and errors
  % Defining widith or errorbars
  pxd = (xub-xlb)*0.015; pyd = (yub-ylb)*0.015; 
  % Plotting crosslines 
  plot([px px],[py-pye py+pye],'k-');
  plot([px-pxe px+pxe],[py py],'k-');
  % Plotting errorbars
  plot([px-pxd px+pxd],[py+pye py+pye],'k-');
  plot([px-pxd px+pxd],[py-pye py-pye],'k-');
  plot([px-pxe px-pxe],[py-pyd py+pyd],'k-');
  plot([px+pxe px+pxe],[py-pyd py+pyd],'k-');
end  
% Finally plotting data points
% LVH data from Bache et al, Cardio Res, 1999
plot(MVO2_st_LVH_z(1:4),PCrATP_st_LVH_z(1:4),'ko',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Setting axis properties
axis([xlb xub ylb yub]);
% set(gca,'XtickLabel',[]);
set(gca,'fontsize',18); 
set(gca,'Xtick',0:2:12);
set(gca,'Ytick',ylb:0.5:yub);
ylabel('CrP/ATP'); 
xlabel('MVO_2 (\mumol min^{-1} (g tissue)^{-1})');

%%
figure(4026); clf;
axes('position',[0.15 0.15 0.75 0.75]);
hold on; box on;
xlb = 0; xub = 13; ylb = -0.025; yub = 0.5;
plot(MVO2_sim_LVH,PIPCr_sim_LVH,'k-','LineWidth',1.5);
% Labeling the graphs
% Zero J_AtC
plot([MVO2_sim_LVH(1) MVO2_sim_LVH(1)],[ylb yub],'k:','LineWidth',1.0);
% Baseline J_AtC
plot([MVO2_BL MVO2_BL],[ylb yub],'k:','LineWidth',1.0);
% Maximum J_AtC
plot([max(MVO2_st_z) max(MVO2_st_z)],[ylb yub],'k:','LineWidth',1.0);
text((xub-xlb)*0.05+xlb,(yub-ylb)*0.9+ylb,'F','FontSize',24);
% plot(MVO2_st_z,PIPCr_inp,'r+');   
% Modify the baseline value of PI/ATP to represent its actual meaning
PIPCr_st_LVH_z([1]) = (1e-3/CrPBL_cellwater_LVH)/2; % average value of the detectable PI/PCr 0.0708
PIPCr_st_LVH_e([1]) = (1e-3/CrPBL_cellwater_LVH)/2;
for i=1:4
  % You can use your dateset instead of MVO2 and PCrATP
  px = MVO2_st_LVH_z(i); pxe = MVO2_st_LVH_e(i); % Loading x-axis data and errors
  py = PIPCr_st_LVH_z(i); pye = PIPCr_st_LVH_e(i); % Loading y-axis data and errors
  % Defining widith or errorbars
  pxd = (xub-xlb)*0.015; pyd = (yub-ylb)*0.015; 
  % Plotting crosslines 
  plot([px px],[py-pye py+pye],'k-');
  plot([px-pxe px+pxe],[py py],'k-');
  % Plotting errorbars
  plot([px-pxd px+pxd],[py+pye py+pye],'k-');
  plot([px-pxd px+pxd],[py-pye py-pye],'k-');
  plot([px-pxe px-pxe],[py-pyd py+pyd],'k-');
  plot([px+pxe px+pxe],[py-pyd py+pyd],'k-');
end  
% plot(MVO2_st_z,PIPCr_inp,'r+');
% plot(MVO2_st_z,PIPCr_st_z,'ks',...
%     'Markersize',8,'MarkerFaceColor',[1 1 1]);
% LVH data from Bache et al, Cardio Res, 1999
plot(MVO2_st_LVH_z(1),PIPCr_st_LVH_z(1),'ko',...
    'Markersize',10,'MarkerFaceColor',[0.5 0.5 0.5]);
% LVH data from Bache et al, Cardio Res, 1999
plot(MVO2_st_LVH_z(2:4),PIPCr_st_LVH_z(2:4),'ko',...
    'Markersize',10,'MarkerFaceColor',[1 1 1]);
% Setting axis properties
axis([xlb xub ylb yub]);
set(gca,'fontsize',18);
set(gca,'Xtick',0:2:12);
set(gca,'Ytick',0:0.1:0.5); %set(gca,'Ytick',ylb:0.1:yub);
% set(gca,'XtickLabel',[]);
xlabel('MVO_2 (\mumol min^{-1} (g tissue)^{-1}');
ylabel('\DeltaPi/CrP'); 



%% Output plots
print -f4021 -r600 -depsc St_MVO2_CrPATP_Control;
print -f4022 -r600 -depsc St_MVO2_PICrP_Control;
print -f4023 -r600 -depsc St_MVO2_CrPATP_mLVH;
print -f4024 -r600 -depsc St_MVO2_PICrP_mLVH;
print -f4025 -r600 -depsc St_MVO2_CrPATP_LVH;
print -f4026 -r600 -depsc St_MVO2_PICrP_LVH;



